#ifndef CController_h__
#define CController_h__

#include "CBase.h"
#include <map>

using namespace std;

class CController : public CBase
{
public:
	typedef pair<int, int> Keycode_Command_Pair;

	class PropertyFlag : public CBase::PropertyFlag
	{
	public:
		enum
		{
			COMMAND_MAP = NUM_OF_CBASE_PROPERTY_FLAG,
			NUM_OF_CCONTROLLER_PROPERTY_FLAG
		};
	};

	enum COMMAND
	{
		PAUSE = 0,
		RESUME
	};

	static CController* addComponentTo(Entity* object);
	static CController* addComponentTo(Entity* object, map<int, int> commandMap);

	int update(float deltaTime);
	virtual int commandProcess(int command);
	virtual ComponentProperty	getProperty(int propertyFlag);

	// To add or remove a control key.
	// pass command as CBase::COMMAND::NULL_COMMAND to remove control key.
	virtual int	setProperty(ComponentProperty propertyData, int propertyFlag) throw(ComponentProperty::InvalidPointerType);

	// To add a new control key.
	// @param keyCode: Virtual keycode of the key.
	//		it's marked as VK_<Keyname>
	//		if it's numpad number: VK_NUMPAD<number>
	//		if it's alphabet or number key then pass UPPER CASE char: 'S'
	// @param command: Entity Command bind with the key.
	int addKey(int keyCode, int command);

	// To erase all control action bind with the key.
	// @param keyCode: Virtual keycode of the key.
	//		it's marked as VK_<Keyname>
	//		if it's numpad number: VK_NUMPAD<number>
	//		if it's alphabet or number key then pass UPPER CASE char: 'S'
	int eraseKey(int keyCode);

private:
	CController();
	~CController();

private:
	map<int, int> m_CommandMap;
	bool	m_isPaused;
};

#endif // CController_h__
