#include "Entity.h"
#include "Config.h"
#include "CAnimation.h"
#include "CPosition.h"

Entity::Entity()
{

}

Entity::~Entity()
{
	m_components.clear();
}

int Entity::update(float delta)
{
	while (true)
	{
		Entity::COMMAND command = popCommand();
		if (command == NULL_COMMAND)
			break;

		commandProcess(command);
	}

	// Component updating.
	for (Pair_ID_ptrCBase componentPair : m_components)
		componentPair.second->update(delta);

	return 0;
}

void Entity::draw(HDC hdc)
{
	
}

void Entity::addComponent(CBase* component)
{
	m_components.insert(Pair_ID_ptrCBase(component->getTypeID(), component));
}

ComponentProperty Entity::getProperty(ID::COMPONENT componentFlag, int propertyFlag) 
throw(ID::Unexpected)
{
	try
	{
		return m_components.at(componentFlag)->getProperty(propertyFlag);
	}
	catch (out_of_range e)
	{
		throw ID::Unexpected();
	}
}

int Entity::setProperty(ID::COMPONENT componentFlag, ComponentProperty propertyData, int propertyFlag) 
throw(ID::Unexpected, ComponentProperty::InvalidPointerType)
{
	try
	{
		m_components.at(componentFlag)->setProperty(propertyData, propertyFlag);
		return FSUCCESS;
	}
	catch (out_of_range e)
	{
		throw ID::Unexpected();
	}
	catch (ComponentProperty::InvalidPointerType e)
	{
		throw e;
	}

	return FFAILD;
}

int Entity::pushCommand(COMMAND command)
{
	m_commandQueue.push_back(command);
	return FSUCCESS;
}

Entity::COMMAND Entity::popCommand()
{
	if (m_commandQueue.empty())
		return NULL_COMMAND;

	Entity::COMMAND result = m_commandQueue.front();
	m_commandQueue.erase(m_commandQueue.begin());
	return result;
}

int Entity::commandProcess(COMMAND command)
{
	for (ReactCommand react : m_reactCommands)
	{
		if (command == react.command)
		{
			m_components.at(react.componentID)->pushCommand(react.reactCommand);
		}
	}

	for (ReactFunction react : m_reactFunctions)
	{
		if (command == react.command)
			react.reactFunction(this);
	}

	return FSUCCESS;
}

int Entity::settingReact(ReactCommand react)
{
	m_reactCommands.push_back(react);
	return FSUCCESS;
}

int Entity::settingReact(ReactFunction react)
{
	m_reactFunctions.push_back(react);
	return FSUCCESS;
}