#ifndef __VECTOR2_H__
#define __VECTOR2_H__

#include "FConfig.h"
#include <map>
#include <exception>

using namespace std;

struct Vector2
{
	int x, y;

	Vector2(int _x = 0, int _y = 0)
	{
		x = _x;
		y = _y;
	}
};

// The struct declared to support component Property's get/set methods
struct ComponentProperty
{
	enum Type
	{
		PTYPE_VOID = 0,
		PTYPE_VECTOR2,
		PTYPE_COMMAND_MAP
	};

	class InvalidPointerType : public std::exception
	{
		virtual const char* what() const throw()
		{
			return "A ComponentProperty type variable has been casted to wrong type.";
		}
	};

	union
	{
		void*			ptrVoid;
		Vector2*		ptrVector2;
		map<int, int>*	ptrCommandMap;
		pair<int, int>* ptrCommandPair;
	};

	bool autoCleanup;
	Type typeFlag;

	// @param _ptrVoid: void type pointer to the data location.
	// @param _typeFlag: an enum to define what is really type of this data location.
	// @param _autoCleanup: define if the passed pointer will be released in the same time with variable.
	// WARNING: In case _typeFlag == PTYPE_VOID and _autoCleanup == true.
	// The very data's destructor will not be called.
	ComponentProperty(	void* _ptrData = nullptr, 
						ComponentProperty::Type _typeFlag = PTYPE_VOID, 
						bool _autoCleanup = false)
	{
		ptrVoid = _ptrData;
		typeFlag = _typeFlag;
		autoCleanup = _autoCleanup;
	}

	ComponentProperty(const ComponentProperty& source)
	{
		ptrVoid = source.ptrVoid;
		typeFlag = source.typeFlag;
		autoCleanup = false;
	}

	~ComponentProperty()
	{
		if (autoCleanup)
		{
			switch (typeFlag)
			{
			// Destructor will not be called.
			case ComponentProperty::PTYPE_VOID:
				SAFE_DELETE(ptrVoid);
				break;

			case ComponentProperty::PTYPE_VECTOR2:
				SAFE_DELETE(ptrVector2);
				break;

			case ComponentProperty::PTYPE_COMMAND_MAP:
				SAFE_DELETE(ptrCommandPair);
				break;

			default:
				break;
			}
		}
	}
};
#endif // __VECTOR2_H__
