#include "Factory.h"

Factory* Factory::m_instance = nullptr;

Factory::Factory()
{}

Factory::~Factory()
{}

void Factory::release()
{
	SAFE_DELETE(m_instance);
}

void Factory::init()
{
	m_instance = new Factory();
}

Factory* Factory::getInstance()
{
	return m_instance;
}

Entity* Factory::create(ID::Entity entityID)
{
	switch (entityID)
	{
	case ID::STDIO_LOGO:				return create_STDIO_LOGO();
	case ID::STDIO_LOGO_CONTROLABLE:	return create_STDIO_LOGO_CONTROLABLE();
	default:							return nullptr;
	}
}


Entity* Factory::create_STDIO_LOGO()
{
	auto entity = new Entity();

	CAnimation::addComponentTo(entity, "Resource\\logo.bmp");
	CPosition::addComponentTo(entity, 250, 250);

	entity->settingReact(Entity::ReactCommand(Entity::COMMAND::HIDE, ID::COMPONENT::ANIMATION, CAnimation::COMMAND::HIDE));
	entity->settingReact(Entity::ReactCommand(Entity::COMMAND::SHOW, ID::COMPONENT::ANIMATION, CAnimation::COMMAND::SHOW));
	entity->settingReact(Entity::ReactFunction(Entity::SHOW, [](Entity* entity)
	{
		ComponentProperty newPos(new Vector2(rand() % 400 + 50, rand() % 400 + 50), ComponentProperty::Type::PTYPE_VECTOR2, true);
		entity->setProperty(ID::COMPONENT::POSITION, newPos, CPosition::PropertyFlag::PFLAG_POSITION);
	}
	));

	return entity;
}

Entity* Factory::create_STDIO_LOGO_CONTROLABLE()
{
	auto entity = new Entity();

	CAnimation::addComponentTo(entity, "Resource\\logo.bmp");
	CPosition::addComponentTo(entity, 50, 50);
	auto controller = CController::addComponentTo(entity);

	entity->settingReact(Entity::ReactCommand(Entity::COMMAND::HIDE, ID::COMPONENT::ANIMATION, CAnimation::COMMAND::HIDE));
	entity->settingReact(Entity::ReactCommand(Entity::COMMAND::SHOW, ID::COMPONENT::ANIMATION, CAnimation::COMMAND::SHOW));
	entity->settingReact(Entity::ReactFunction(Entity::SHOW, [](Entity* entity)
	{
		ComponentProperty newPos(new Vector2(rand() % 400 + 50, rand() % 400 + 50), ComponentProperty::Type::PTYPE_VECTOR2, true);
		entity->setProperty(ID::COMPONENT::POSITION, newPos, CPosition::PropertyFlag::PFLAG_POSITION);
	}
	));

	controller->addKey('S', Entity::COMMAND::SHOW);
	controller->addKey('H', Entity::COMMAND::HIDE);

	return entity;
}